﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Garage
{
    public partial class CarDialog : Form
    {
        private Car _car;

        public Car Car
        {
            get
            {
                return _car;
            }
            set
            {
                _car = value;

                manufacturerComboBox.Text = _car.Manufacturer;
                nameTextBox.Text = _car.Name;
                licensePlateTextBox.Text = _car.LicensePlate;
                colourButton.BackColor = _car.Colour;
                priceNumericUpDown.Value = _car.Price;
                fuelNumericUpDown.Value = _car.Fuel;

                if (_car.Tyres == TyreType.Summer)
                {
                    summerTiresRadioButton.Checked = true;
                }
                else
                {
                    winterTiresRadioButton.Checked = true;
                }

                manufacturerComboBox.Enabled = false;
                licensePlateLabel.Enabled = false;
                priceNumericUpDown.Enabled = false;
            }
        }

        public CarDialog()
        {
            InitializeComponent();
        }

        private void manufacturerComboBox_Validating(object sender, CancelEventArgs e)
        {
            if (manufacturerComboBox.Items.Contains(manufacturerComboBox.Text))
            {
                errorProvider.SetError(manufacturerComboBox, null);
                e.Cancel = false;
            }
            else
            {
                errorProvider.SetError(manufacturerComboBox, "Please select an existing manufacturer.");
                e.Cancel = true;
            }
        }

        private void nameTextBox_Validating(object sender, CancelEventArgs e)
        {
            if (String.IsNullOrWhiteSpace(nameTextBox.Text))
            {
                errorProvider.SetError(nameTextBox, "Name cannot be blank.");
                e.Cancel = true;
            }
            else
            {
                errorProvider.SetError(nameTextBox, null);
                e.Cancel = false;
            }
        }

        private void licensePlateTextBox_Validating(object sender, CancelEventArgs e)
        {
            if (Regex.IsMatch(licensePlateTextBox.Text,
                    "^[A-Z0-9][A-Z][A-Z0-9] [0-9]{4}$"))
            {
                errorProvider.SetError(licensePlateTextBox, null);
                e.Cancel = false;
            }
            else
            {
                errorProvider.SetError(licensePlateTextBox, "License Plate should be in the following format: XLX DDDD, where X is alphanumeric, L is a letter and D is a digit with all letters uppercase.");
                e.Cancel = true;
            }

            //if (licensePlateTextBox.Text.Length != 8)
            //{
            //    errorProvider.SetError(licensePlateTextBox, "License Plate should be 8 characters long, including spaces.");
            //    e.Cancel = true;
            //    return;
            //}
            //
            //if (!Char.IsLetterOrDigit(licensePlateTextBox.Text[0]) ||
            //    !Char.IsLetterOrDigit(licensePlateTextBox.Text[2]) ||
            //    !Char.IsLetter(licensePlateTextBox.Text[1]) ||
            //    (licensePlateTextBox.Text[3] != ' ') ||
            //    licensePlateTextBox.Text.Skip(4)
            //       .Any(ch => !Char.IsDigit(ch)))
            //{
            //    errorProvider.SetError(licensePlateTextBox, "License Plate should be in the following format: XLX DDDD, where X is alphanumeric, L is a letter and D is a digit.");
            //    e.Cancel = true;
            //}
            //else
            //{
            //    if (licensePlateLabel.Text.Any(ch => Char.IsLower(ch)))
            //    {
            //        errorProvider.SetError(licensePlateTextBox, "All letters should be uppercase.");
            //        e.Cancel = true;
            //    }
            //    else
            //    {
            //        errorProvider.SetError(licensePlateTextBox, null);
            //        e.Cancel = false;
            //    }
            //}
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void confirmButton_Click(object sender, EventArgs e)
        {
            if (ValidateChildren())
            {
                _car = new Car(manufacturerComboBox.Text,
                    licensePlateTextBox.Text,
                    priceNumericUpDown.Value);

                _car.Name = nameTextBox.Text;
                _car.Colour = colourButton.BackColor;
                _car.Fuel = fuelNumericUpDown.Value;
                
                if (summerTiresRadioButton.Checked)
                {
                    _car.Tyres = TyreType.Summer;
                }
                else
                {
                    _car.Tyres = TyreType.Winter;
                }

                DialogResult = DialogResult.OK;
                Close();
            }
        }

        private void colourButton_Click(object sender, EventArgs e)
        {
            ColorDialog diag = new ColorDialog();
            diag.Color = colourButton.BackColor;

            if (diag.ShowDialog() == DialogResult.OK)
            {
                colourButton.BackColor = diag.Color;
            }
        }
    }
}
